#!/usr/bin/env node

'use strict';
const pkg = require('../package.json');
const path = require('path');
const assert = require('assert-plus');
const program = require('commander');
const chalk = require('chalk');
const AWS = require('aws-sdk');
const Deploy = require('../lib').Deploy;

const showHelp = () => {
  program.outputHelp(chalk.blue);
};

function exitIfFailed(fn) {
  const args = Array.prototype.slice.call(arguments, 1);
  try {
    return fn.apply(null, args);
  } catch (err) {
    console.error(chalk.red(err.message));
    showHelp();
    process.exit(1);
  }
}

const exitOnFailedPromise = (promise) => promise.catch(err => {
  console.error(chalk.red(err.message));
  showHelp();
  process.exit(1);
});

const bold = chalk.bold.white;

const getParams = (options, service, cluster, image) => {
  const accessKey = options.accessKeyId;
  const secretKey = options.secretAccessKey;
  const region = options.region;

  assert.string(service, 'service is required');
  assert.string(cluster, 'cluster is required');
  assert.string(image, 'image is required');

  return {
    accessKeyId: accessKey,
    secretAccessKey: secretKey,
    region,
    taskFile: options.taskFile,
    envFile: options.envFile,
    service,
    cluster,
    image,
    deploymentOptions: {
      desiredCount: options.desiredCount,
      min: options.min,
      max: options.max
    }
  };
};

const run = (params) => {

  const config = {
    apiVersion: '2014-11-13',
    accessKeyId: params.accessKeyId,
    secretAccessKey: params.secretAccessKey,
    region: params.region
  };
  const ecs = new AWS.ECS(config);

  const deploy = Deploy.create(ecs);
  console.log(chalk.blue(`Deploying ${bold(params.service)} to ${bold(params.cluster)}`));
  const taskFile = params.taskFile ? path.resolve(params.taskFile) : params.taskFile;
  const envFile = params.envFile ? path.resolve(params.envFile): params.envFile;
  return deploy.updateWithFile(params.service, params.cluster, params.image, params.deploymentOptions, taskFile, envFile);
};

program
  .version(pkg.version)
  .option('-k, --access-key-id <id>', 'AWS Access key ID. Env: $AWS_ACCESS_KEY_ID')
  .option('-s, --secret-access-key <secret>', 'AWS Secret Access Key. Env: $AWS_SECRET_ACCESS_KEY')
  .option('-r, --region <region>', 'AWS Region. Env: $AWS_REGION')
  .option('-c, --desired-count <count>', 'Desired count for service.')
  .option('-m, --min <min>', 'Minimum healthy percent for service when deploying. Default: 100')
  .option('-M, --max <max>', 'Maximum Percent of healthy tasks running when deploying. Default: 200')
  .option('-w, --task-file <taskFile>', 'path to Task Definition file')
  .option('-e, --env-file <envFile>', 'path to env file to use for Task Definition environment variables');

program
  .command('deploy [service] [cluster] [image]')
  .description('Update ECS Service and Task Definition')
  .action((service, cluster, image) => {
    const params = exitIfFailed(getParams, program, service, cluster, image);
    exitOnFailedPromise(run(params));
  });

program.parse(process.argv);

if (!process.argv.slice(2).length) {
  showHelp();
}
