'use strict';
const Task = require('./task');
const Service = require('./service');
const util = require('./util');
const fs = require('mz/fs');
const chalk = require('chalk');
const template = require('./template');
const assert = require('assert-plus');

const logInfo = (message) => {
  console.log(chalk.blue(message));
};

const logSuccess = (message) => {
  console.log(chalk.green(message));
};

const bold = chalk.bold.white;

const getFileContent = (file) => {
  return fs.readFile(file, 'utf-8').catch(() => {
    throw new Error(`Could not find file ${file}`);
  });
};

const create = (ecs) => {

  const service = Service.create(ecs);
  const task = Task.create(ecs);

  const update = (name, cluster, image, deploymentOptions, taskOverrides, env) => {
    logInfo(`Getting Task associated with ${bold(name)}`);
    return service.get(name, cluster)
      .then(serviceDef => {
        const taskDef = util.parseTaskArn(serviceDef.taskDefinition);
        logInfo(`Updating task: ${bold(taskDef.family)}`);
        return task.update(taskDef.family, image, taskOverrides, env).then(() => [serviceDef, taskDef]);
      })
      .then(params => {
        const [serviceDef, taskDef] = params;
        logSuccess(`Updated Task: ${bold(taskDef.family)}`);

        const desiredCount = deploymentOptions.desiredCount || serviceDef.desiredCount;
        const minimumHealthyPercent = deploymentOptions.min || 100;
        const maximumPercent = deploymentOptions.max || 200;
        const serviceParams = {
          service: name,
          cluster,
          taskDefinition: taskDef.family,
          desiredCount,
          deploymentConfiguration: {
            maximumPercent,
            minimumHealthyPercent
          }
        };

        logInfo(`Updating service ${bold(name)} to count ${bold(serviceParams.desiredCount)}`);
        return service.update(serviceParams);
      })
      .then(() => {
        logSuccess('Updated Service');
        logInfo('Waiting for service to become stable');
        return service.waitForDeployment(name, cluster);
      })
      .then(() => logSuccess(`Done deploying ${bold(name)}`));
  };

  const updateWithFile = (name, cluster, image, deploymentOptions, taskFile, envFile) => {

    const taskFileContent = (taskFile) ? getFileContent(taskFile) : Promise.resolve('{}');
    const envFileContent = (envFile) ? getFileContent(envFile) : Promise.resolve('');

    const taskDefPromise = taskFileContent.then(template.process);
    const envDefPromise = envFileContent.then(util.parseEnv);

    return Promise.all([taskDefPromise, envDefPromise])
      .then(params => {
        const [taskDef, envDef] = params;
        return update(name, cluster, image, deploymentOptions, taskDef, envDef);
      });
  };

  return {
    update,
    updateWithFile
  };
};

module.exports = {
  create
};
