'use strict';
const fs = require('mz/fs');
const path = require('path');
const assert = require('assert-plus');
const _ = require('lodash');

const normalizeTask = (task) => {
  const taskDef = _.cloneDeep(task);

  // remove fields which are no specified during task registration
  delete taskDef.taskDefinitionArn;
  delete taskDef.status;
  delete taskDef.requiresAttributes;
  delete taskDef.revision;

  return taskDef;
};

const create = (ecs) => {
  const get = ({family, revision}) => {
    const taskName = revision ? `${family}:${revision}` : family;
    return ecs.describeTaskDefinition({taskDefinition: taskName}).promise().then(t => t.taskDefinition);
  };

  const saveTask = (task, toPath) => {
    const taskDef = normalizeTask(task);

    for (let i in taskDef.containerDefinitions) {
      delete taskDef.containerDefinitions[i].image;
    }

    const taskFilePath = path.join(toPath, `${taskDef.family}.json`);
    const taskJson = JSON.stringify(taskDef, null, 4);
    return fs.writeFile(taskFilePath, taskJson);
  };

  const saveAsJson = (family, path) => {
    return get({family}).then(task => saveTask(task, path));
  };

  const update = (family, image, overrides, env) => {
    assert.string(family, 'family is required');
    assert.string(image, 'image is required');

    return get({family})
      .then(normalizeTask)
      .then(task => {
        const finalTask = overrides ? _.merge(task, overrides) : task;

        const containers = finalTask.containerDefinitions;

        assert.bool(containers.length > 0, 'task must have container definitions');

        const container = containers[0];
        container.image = image;
        container.environment = (env && env.length) ? env: container.environment;

        return ecs.registerTaskDefinition(finalTask).promise();
      });
  };

  return {
    get,
    saveAsJson,
    update
  };
};

module.exports = {
  create
};
